@@ -49,27 +49,15 @@ class DatabaseFeatures(BaseDatabaseFeatures):
         END;
     $$ LANGUAGE plpgsql;"""
     supports_over_clause = True
+    supports_aggregate_filter_clause = True
+    supported_explain_formats = {'JSON', 'TEXT', 'XML', 'YAML'}
+    validates_explain_options = False  # A query will error on invalid options.
 
     @cached_property
-    def supports_aggregate_filter_clause(self):
-        return self.connection.pg_version >= 90400
-
-    @cached_property
-    def has_select_for_update_skip_locked(self):
+    def is_postgresql_9_5(self):
         return self.connection.pg_version >= 90500
 
-    @cached_property
-    def has_brin_index_support(self):
-        return self.connection.pg_version >= 90500
-
-    @cached_property
-    def has_jsonb_datatype(self):
-        return self.connection.pg_version >= 90400
-
-    @cached_property
-    def has_jsonb_agg(self):
-        return self.connection.pg_version >= 90500
-
-    @cached_property
-    def has_gin_pending_list_limit(self):
-        return self.connection.pg_version >= 90500
+    has_select_for_update_skip_locked = is_postgresql_9_5
+    has_brin_index_support = is_postgresql_9_5
+    has_jsonb_agg = is_postgresql_9_5
+    has_gin_pending_list_limit = is_postgresql_9_5
