@@ -9,19 +9,14 @@ They also act as reverse fields for the purposes of the Meta API because
 they're the closest concept currently available.
 """
 
-from __future__ import unicode_literals
-
-import warnings
-
 from django.core import exceptions
-from django.utils.deprecation import RemovedInDjango20Warning
-from django.utils.encoding import smart_text
 from django.utils.functional import cached_property
 
 from . import BLANK_CHOICE_DASH
+from .mixins import FieldCacheMixin
 
 
-class ForeignObjectRel(object):
+class ForeignObjectRel(FieldCacheMixin):
     """
     Used by ForeignObject to store information about the relation.
 
@@ -56,14 +51,6 @@ class ForeignObjectRel(object):
     # __init__ as the field doesn't have its model yet. Calling these methods
     # before field.contribute_to_class() has been called will result in
     # AttributeError
-    @property
-    def to(self):
-        warnings.warn(
-            "Usage of ForeignObjectRel.to attribute has been deprecated. "
-            "Use the model attribute instead.",
-            RemovedInDjango20Warning, 2)
-        return self.model
-
     @cached_property
     def hidden(self):
         return self.is_hidden()
@@ -79,7 +66,7 @@ class ForeignObjectRel(object):
     @property
     def target_field(self):
         """
-        When filtering against this relation, returns the field on the remote
+        When filtering against this relation, return the field on the remote
         model against which the filtering should happen.
         """
         target_fields = self.get_path_info()[-1].target_fields
@@ -129,18 +116,18 @@ class ForeignObjectRel(object):
 
     def get_choices(self, include_blank=True, blank_choice=BLANK_CHOICE_DASH):
         """
-        Return choices with a default blank choices included, for use as
-        SelectField choices for this field.
+        Return choices with a default blank choices included, for use
+        as <select> choices for this field.
 
         Analog of django.db.models.fields.Field.get_choices(), provided
         initially for utilization by RelatedFieldListFilter.
         """
         return (blank_choice if include_blank else []) + [
-            (x._get_pk_val(), smart_text(x)) for x in self.related_model._default_manager.all()
+            (x.pk, str(x)) for x in self.related_model._default_manager.all()
         ]
 
     def is_hidden(self):
-        "Should the related object be hidden?"
+        """Should the related object be hidden?"""
         return bool(self.related_name) and self.related_name[-1] == '+'
 
     def get_joining_columns(self):
@@ -176,11 +163,15 @@ class ForeignObjectRel(object):
             return self.related_name
         return opts.model_name + ('_set' if self.multiple else '')
 
-    def get_cache_name(self):
-        return "_%s_cache" % self.get_accessor_name()
+    def get_path_info(self, filtered_relation=None):
+        return self.field.get_reverse_path_info(filtered_relation)
 
-    def get_path_info(self):
-        return self.field.get_reverse_path_info()
+    def get_cache_name(self):
+        """
+        Return the name of the cache key to use for storing an instance of the
+        forward model on the reverse model.
+        """
+        return self.get_accessor_name()
 
 
 class ManyToOneRel(ForeignObjectRel):
@@ -200,7 +191,7 @@ class ManyToOneRel(ForeignObjectRel):
 
     def __init__(self, field, to, field_name, related_name=None, related_query_name=None,
                  limit_choices_to=None, parent_link=False, on_delete=None):
-        super(ManyToOneRel, self).__init__(
+        super().__init__(
             field, to,
             related_name=related_name,
             related_query_name=related_query_name,
@@ -239,7 +230,7 @@ class OneToOneRel(ManyToOneRel):
 
     def __init__(self, field, to, field_name, related_name=None, related_query_name=None,
                  limit_choices_to=None, parent_link=False, on_delete=None):
-        super(OneToOneRel, self).__init__(
+        super().__init__(
             field, to, field_name,
             related_name=related_name,
             related_query_name=related_query_name,
@@ -262,7 +253,7 @@ class ManyToManyRel(ForeignObjectRel):
     def __init__(self, field, to, related_name=None, related_query_name=None,
                  limit_choices_to=None, symmetrical=True, through=None,
                  through_fields=None, db_constraint=True):
-        super(ManyToManyRel, self).__init__(
+        super().__init__(
             field, to,
             related_name=related_name,
             related_query_name=related_query_name,
