@@ -12,12 +12,12 @@ class Index:
     # cross-database compatibility with Oracle)
     max_name_length = 30
 
-    def __init__(self, *, fields=[], name=None, db_tablespace=None):
-        if not isinstance(fields, list):
-            raise ValueError('Index.fields must be a list.')
+    def __init__(self, *, fields=(), name=None, db_tablespace=None):
+        if not isinstance(fields, (list, tuple)):
+            raise ValueError('Index.fields must be a list or tuple.')
         if not fields:
             raise ValueError('At least one field is required to define an index.')
-        self.fields = fields
+        self.fields = list(fields)
         # A list of 2-tuple with the field name and ordering ('' or 'DESC').
         self.fields_orders = [
             (field_name[1:], 'DESC') if field_name.startswith('-') else (field_name, '')
@@ -69,8 +69,8 @@ class Index:
 
     def clone(self):
         """Create a copy of this Index."""
-        path, args, kwargs = self.deconstruct()
-        return self.__class__(*args, **kwargs)
+        _, _, kwargs = self.deconstruct()
+        return self.__class__(**kwargs)
 
     @staticmethod
     def _hash_generator(*args):
